
#include "channel.h"

#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

ChannelItem *rss_channel_item_new(gchar *title, gchar *url, gchar *desc)
{
    ChannelItem *ret = g_new0(ChannelItem, 1);

    ret->title = g_strdup(title);
    ret->url = g_strdup(url);
    ret->desc = g_strdup(desc);

    return ret;
}


void rss_channel_item_free(ChannelItem *item)
{
    g_free(item->title);
    g_free(item->url);
    g_free(item->desc);
    g_free(item);
}


Channel *rss_channel_new(gchar *title, gchar *url, gchar *desc)
{
    Channel *ret = g_new0(Channel, 1);

    ret->title = g_strdup(title);
    ret->url = g_strdup(url);
    ret->desc = g_strdup(desc);

    //ret->items = g_ptr_array_new();
    ret->count = 0;

    return ret;
}

Channel *rss_channel_new_copy(Channel *c)
{
    guint i;
    Channel *ret = g_new0(Channel, 1);

    ret->title = g_strdup(c->title);
    ret->url = g_strdup(c->url);
    ret->desc = g_strdup(c->desc);

    for(i=0;i<c->count;i++)
    {
        ChannelItem *item = rss_channel_get_item(c, i);
	rss_channel_add_item(ret, rss_channel_item_new(item->title, item->url, item->desc));
    }

    return ret;
}

void rss_channel_free(Channel *channel)
{
    g_free(channel->title);
    g_free(channel->url);
    g_free(channel->desc);
    rss_channel_clear(channel);
    g_free(channel);
}

void rss_channel_clear(Channel *channel)
{
    guint i;
    for(i=0;i<channel->count;i++)
        rss_channel_item_free(channel->items[i]);
    channel->count = 0;

    //while(channel->items->len>0)
    //    rss_channel_item_free(g_ptr_array_remove_index(channel->items, 0));
}

void rss_channel_add_item(Channel *channel, ChannelItem *item)
{
    //g_ptr_array_add(channel->items, item);
    channel->items[channel->count++] = item;
}

guint rss_channel_items_count(Channel *channel)
{
    return channel->count;
}

ChannelItem *rss_channel_get_item(Channel *channel, guint idx)
{
    //return g_ptr_array_index(channel->items, idx);
    return channel->items[idx];
}


void rss_channel_update(Channel *channel)
{
    rss_channel_clear(channel);

    xmlDocPtr doc = xmlParseFile(channel->url);
    if(doc == NULL)
    {
	//std::cerr << "Cannot read file: " << _title << std::endl;
	return;
    }

    xmlNodePtr cur = xmlDocGetRootElement(doc);
    if (cur == NULL)
    {
	//std::cerr << "Empty document: " << _title << std::endl;
	xmlFreeDoc(doc);
	return;
    }

    gboolean rdf = FALSE;
    // We must determine whether the document is RDF or RSS since they
    // have slightly different structures. We handle both.
    if(!xmlStrcmp(cur->name, (const xmlChar *)"rss"))
	rdf = FALSE;
    else if(!xmlStrcmp(cur->name, (const xmlChar *)"rdf") ||
	    !xmlStrcmp(cur->name, (const xmlChar *)"RDF"))
	rdf = TRUE;
    else
    {
	//std::cerr << "Bad document type -- not RSS or RDF." << std::endl;
	xmlFreeDoc(doc);
	return;
    }

    cur = cur->xmlChildrenNode;
    while (cur && xmlIsBlankNode(cur))
	cur = cur->next;
    if (cur == 0)
	return;

    if (xmlStrcmp(cur->name, (const xmlChar *)"channel"))
    {
	//std::cerr
	//    << "Bad document: did not immediately find the channel element."
	//   << std::endl;
	//free(this);
	return;
    }

    if (!rdf)
    {
	//std::cerr << "document is RSS" << std::endl;
	cur = cur->xmlChildrenNode;
    }

    while (cur != NULL)
    {
	if((!xmlStrcmp(cur->name, (const xmlChar *)"title")))
	{
	    g_free(channel->title);
	    channel->title = g_strdup(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
	    //set_title((char *)xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
	}
	else if((!xmlStrcmp(cur->name, (const xmlChar *)"description")))
	{
	    g_free(channel->desc);
	    channel->desc = g_strdup(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
	    //set_description((char *)xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
	}
	else if((!xmlStrcmp(cur->name, (const xmlChar *)"channel")) && rdf)
	{
	    xmlNodePtr pcur = cur->xmlChildrenNode;
	    while (pcur != NULL)
	    {
		if((!xmlStrcmp(pcur->name, (const xmlChar *)"title")))
		{
		    g_free(channel->title);
		    channel->title = g_strdup(xmlNodeListGetString(doc, pcur->xmlChildrenNode, 1));
		    //set_title((char *)xmlNodeListGetString(doc, pcur->xmlChildrenNode, 1));
		}
		else if((!xmlStrcmp(pcur->name, (const xmlChar *)"description")))
		{
                    g_free(channel->desc);
		    channel->desc = g_strdup(xmlNodeListGetString(doc, pcur->xmlChildrenNode, 1));
		    //set_description((char *)xmlNodeListGetString(doc, pcur->xmlChildrenNode, 1));
		}
		pcur = pcur->next;
	    }
	}
	else if((!xmlStrcmp(cur->name, (const xmlChar *)"item")))
	{
	    ChannelItem *item = rss_channel_item_new("", "", "");

	    xmlNodePtr pcur = cur->xmlChildrenNode;
	    while (pcur != NULL)
	    {
		if ((!xmlStrcmp(pcur->name, (const xmlChar *)"title")))
		{
		    g_free(item->title);
		    item->title = g_strdup(xmlNodeListGetString(doc, pcur->xmlChildrenNode, 1));
		    //item.set_title((char *)xmlNodeListGetString(doc, pcur->xmlChildrenNode, 1));
		}
		if ((!xmlStrcmp(pcur->name, (const xmlChar *)"link")))
		{
                    g_free(item->url);
		    item->url = g_strdup(xmlNodeListGetString(doc, pcur->xmlChildrenNode, 1));
		    //item.set_url((char *)xmlNodeListGetString(doc, pcur->xmlChildrenNode, 1));
		}
		if ((!xmlStrcmp(pcur->name, (const xmlChar *)"description")))
		{
                    g_free(item->desc);
		    item->desc = g_strdup(xmlNodeListGetString(doc, pcur->xmlChildrenNode, 1));
		    //item.set_description((char *)xmlNodeListGetString(doc, pcur->xmlChildrenNode, 1));
		}
		pcur = pcur->next;
	    }


            rss_channel_add_item(channel, item);
	    //items.push_back(item);
	    //item.debug_print();
	}
	cur = cur->next;
    }

}


void _rss_str_append(gchar **dst, gchar *str)
{
    gchar *ret = g_strdup_printf("%s%s", *dst, str);

    g_free(*dst);

    *dst = ret;
}

void rss_channel_generate_html(Channel *channel)
{
    guint i, co;
    gchar *ret = g_strdup("<html><body>");;

    _rss_str_append(&ret, "<font size=7>");
    _rss_str_append(&ret, channel->title);
    _rss_str_append(&ret, "</font><br>");

    _rss_str_append(&ret, "");
    _rss_str_append(&ret, channel->desc);
    _rss_str_append(&ret, "<hr>");

    _rss_str_append(&ret, "<ul>");

    co = rss_channel_items_count(channel);

    for(i=0;i<co;i++)
    {
        ChannelItem *item = rss_channel_get_item(channel, i);

        _rss_str_append(&ret, "<p>");
        _rss_str_append(&ret, "<font size=4><a href=\"");
        _rss_str_append(&ret, item->url);
        _rss_str_append(&ret, "\">");
        _rss_str_append(&ret, item->title);
        _rss_str_append(&ret, "</a></font><br>");
        _rss_str_append(&ret, item->desc);
        _rss_str_append(&ret, "</p><hr><br>");

    }

    _rss_str_append(&ret, "</ul></body></html>");

    if(channel->html != NULL)
        g_free(channel->html);
    channel->html = ret;
    channel->have_html = TRUE;
}

gchar *rss_channel_html(Channel *channel)
{
    if(channel->have_html == FALSE)
        rss_channel_generate_html(channel);
    return channel->html;
}


