
#include <glib.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <pthread.h>

#ifdef HAVE_CONFIG_H
# include "../grr_config.h"
#endif

#include "internal.h"

#include "conversation.h"
#include "debug.h"
#include "gtkconv.h"
#include "gtkplugin.h"
#include "gtkprefs.h"
#include "gtkutils.h"
#include "prefs.h"
#include "notify.h"

#include "channel.h"

#include "channels.h"


static GList *channel_list = NULL;
static GHashTable *channel_hash = NULL;

static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;

static gboolean running = TRUE;

static guint timeout = 10;

gboolean _need_update = FALSE;

guint get_timeout();

void channels_lock()
{
    pthread_mutex_lock(&mutex);
}

void channels_unlock()
{
    pthread_mutex_unlock(&mutex);
}

gboolean need_update()
{
    gboolean ret = FALSE;
    pthread_mutex_lock(&mutex);
    if(_need_update)
        ret = TRUE;
    _need_update = FALSE;
    pthread_mutex_unlock(&mutex);
    return ret;
}

void update_immediately()
{
    pthread_mutex_lock(&mutex);
    _need_update = TRUE;
    pthread_mutex_unlock(&mutex);
}


void read_feed_desc()
{
    GList *feeds = NULL;

    pthread_mutex_lock(&mutex);

    if(channel_list!=NULL)
    {
	GList *d = channel_list;
	while(d!=NULL)
	{
	    rss_channel_free(d->data);
	    d = g_list_next(d);
	}
	g_list_free(channel_list);
	channel_list = NULL;
    }

    feeds = gaim_prefs_get_string_list("/plugins/gtk/gaim-rss-reader/feeds");

    if(feeds)
    {
	while(feeds!=NULL)
	{
	    Channel *c = NULL;
	    gchar **ip = g_strsplit(feeds->data, ";", 2);
	    c = rss_channel_new(ip[0], ip[1], "");

	    channel_list = g_list_append (channel_list, c);

	    feeds = g_list_next(feeds);
	    g_strfreev(ip);
	}
    }

    pthread_mutex_unlock(&mutex);
}

GList* get_channel_list()
{
    return channel_list;
}


Channel* get_channel_by_url(gchar *url)
{
    Channel *ret = NULL;

    pthread_mutex_lock(&mutex);

    if(channel_hash==NULL)
	channel_hash = g_hash_table_new(g_str_hash, g_str_equal);
    else
	ret = g_hash_table_lookup(channel_hash, url);

    if(ret!=NULL)
	ret = rss_channel_new_copy(ret);

    pthread_mutex_unlock(&mutex);

    return ret;
}


void update_channel(Channel *c)
{
    Channel *o = NULL;

    pthread_mutex_lock(&mutex);

    if(channel_hash==NULL)
        channel_hash = g_hash_table_new(g_str_hash, g_str_equal);

    o = g_hash_table_lookup(channel_hash, c->url);
    if(o!=NULL)
	rss_channel_free(o);

    g_hash_table_replace(channel_hash, g_strdup(c->url), c);

    pthread_mutex_unlock(&mutex);
}


gpointer _updater_thread(gpointer data)
{
    while(running)
    {
	GList *clist = NULL;
        GList *d = NULL;
        guint co;

	//g_print("copying clist...\n");

	pthread_mutex_lock(&mutex);
	d = channel_list;
	while(d!=NULL)
	{
	    Channel *c = d->data;
	    clist = g_list_append (clist,
				   rss_channel_new(
						   c->title,
						   c->url,
						   "")
				  );
	    d = g_list_next(d);
	}
        pthread_mutex_unlock(&mutex);

        //g_print("updating channels...\n");

	d = clist;
	while(d!=NULL)
	{
            
	    //g_print("updating \"%s\"\n", c->title);
	    rss_channel_update(d->data);
	    update_channel(d->data);
	    d = g_list_next(d);
	}

	
	//g_print("sleeping...\n");
        set_timeout(gaim_prefs_get_int("/plugins/gtk/gaim-rss-reader/update_time"));

        for(co=0;co<get_timeout();co++)
        {
            g_usleep(G_USEC_PER_SEC/2);
            if(need_update())
                break;
        }
        //g_print("sleeping DONE\n");
    }
    return NULL;
}

gpointer _single_updater_thread(gpointer data)
{
    Channel *c = (Channel *)data;

    rss_channel_update(c);
    update_channel(c);

    return NULL;
}


void start_updater()
{
    pthread_t pt;
    pthread_create(&pt,NULL,_updater_thread, NULL);
}

void start_single_updater(gchar *name, gchar *url)
{
    Channel *c1 = rss_channel_new(name, url, "");
    pthread_t pt;

    pthread_create(&pt,NULL,_single_updater_thread, c1);
}


void set_timeout(guint atimeout)
{
    pthread_mutex_lock(&mutex);
    timeout = atimeout;
    pthread_mutex_unlock(&mutex);
}

guint get_timeout()
{
    guint ret;
    pthread_mutex_lock(&mutex);
    ret = timeout;
    pthread_mutex_unlock(&mutex);
    return ret;
}

