
#include <glib.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>

static GtkTreeModel *
create_and_fill_model (void)
{
    GtkListStore *store;
    GtkTreeIter  iter;

    store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);

    gtk_list_store_append(store, &iter);
    gtk_list_store_set(store, &iter, 0, "Title1", 1, "Url1", -1);

    gtk_list_store_append(store, &iter);
    gtk_list_store_set(store, &iter, 0, "Title2", 1, "Url2", -1);

    return GTK_TREE_MODEL(store);
}


static GtkWidget *
create_list_view_and_model (void)
{
    GtkTreeViewColumn   *col;
    GtkCellRenderer     *renderer;
    GtkWidget           *view;
    GtkTreeModel        *model;
    GtkWidget		*scrolled;

    view = gtk_tree_view_new();

    scrolled = gtk_scrolled_window_new ( NULL, NULL );
    gtk_scrolled_window_set_policy ( GTK_SCROLLED_WINDOW ( scrolled ),
                                    GTK_POLICY_AUTOMATIC,
                                    GTK_POLICY_AUTOMATIC);
    gtk_container_add ( GTK_CONTAINER ( scrolled ),
                       GTK_WIDGET ( view ) );

    /* --- Column #1 --- */

    col = gtk_tree_view_column_new();

    gtk_tree_view_column_set_title(col, "Title");

    /* pack tree view column into tree view */
    gtk_tree_view_append_column(GTK_TREE_VIEW(view), col);

    renderer = gtk_cell_renderer_text_new();

    /* pack cell renderer into tree view column */
    gtk_tree_view_column_pack_start(col, renderer, TRUE);

    /* connect 'text' property of the cell renderer to
     *  model column that contains the first name */
    gtk_tree_view_column_add_attribute(col, renderer, "text", 0);

    g_object_set(renderer,
                 "weight", PANGO_WEIGHT_BOLD,
                 "weight-set", TRUE,
                 NULL);

    /* --- Column #2 --- */

    col = gtk_tree_view_column_new();

    gtk_tree_view_column_set_title(col, "Url");

    /* pack tree view column into tree view */
    gtk_tree_view_append_column(GTK_TREE_VIEW(view), col);

    renderer = gtk_cell_renderer_text_new();

    /* pack cell renderer into tree view column */
    gtk_tree_view_column_pack_start(col, renderer, TRUE);

    /* connect 'text' property of the cell renderer to
     *  model column that contains the last name */
    gtk_tree_view_column_add_attribute(col, renderer, "text", 1);

    model = create_and_fill_model();

    gtk_tree_view_set_model(GTK_TREE_VIEW(view), model);

    g_object_unref(model); /* destroy model automatically with view */

    gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(view)),
                                GTK_SELECTION_SINGLE);

    return scrolled;
}

static GtkWidget *
create_view_and_model (void)
{
    return create_list_view_and_model();
}





int main(int argc, char **argv)
{

    GtkWidget *window;
    GtkWidget *view;

    gtk_init(&argc, &argv);

    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    g_signal_connect(window, "delete_event", gtk_main_quit, NULL); /* dirty */

    view = create_view_and_model();

    gtk_container_add(GTK_CONTAINER(window), view);

    gtk_widget_show_all(window);

    gtk_main();

    return 0;
}
